/*
MainActivity.java was created by Alex Dowlding
 */
package com.example.alex.liftcoach;

import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.BottomNavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AppCompatActivity;
import android.view.MenuItem;
import android.support.v7.preference.PreferenceFragmentCompat;

public class MainActivity extends AppCompatActivity
        implements homeFragment.OnFragmentInteractionListener, editWorkoutFragment.OnFragmentInteractionListener, editWorkoutFragment.OnEditButtonSelectedListener, changeWorkoutFragment.OnFragmentInteractionListener{

    private Fragment fragment;
    private FragmentManager fragmentManager;
    private FragmentTransaction transaction;
    private PreferenceFragmentCompat prefFrag;
    private BottomNavigationView.OnNavigationItemSelectedListener mOnNavigationItemSelectedListener
            = new BottomNavigationView.OnNavigationItemSelectedListener() {

        private Class fragmentClass;
        @Override
        public boolean onNavigationItemSelected(@NonNull MenuItem item) {
            switch (item.getItemId()) {
                case R.id.navigation_home: //they selected the home tab
                    fragmentClass = homeFragment.class;//make the generic fragment the home fragment
                    try {
                        fragment = (Fragment) fragmentClass.newInstance();//new instance of home frag
                        Bundle bundle = new Bundle();//in case we need to send parameters to frag
                        setFragment(fragment, bundle);//set the frag
                    } catch (Exception e) {
                        e.printStackTrace();
                        return false;
                    }
                    return true;
                case R.id.navigation_edit_workouts: //they selected workouts tab
                    fragmentClass = editWorkoutFragment.class;//make the generic fragment the edit frag
                    try {
                        fragment = (Fragment) fragmentClass.newInstance();//new instance of edit frag
                        Bundle bundle = new Bundle();//in case of parameters
                        setFragment(fragment, bundle);//set the frag
                    } catch (Exception e) {
                        e.printStackTrace();
                        return false;
                    }
                    return true;
                case R.id.navigation_settings://they selected the settings tab
                    fragmentClass = preferenceFrag.class;//make the generic frag the settings frag
                    try {
                        prefFrag = (PreferenceFragmentCompat) fragmentClass.newInstance();//new instance
                        Bundle bundle = new Bundle();//in case of parameters
                        setPrefFrag(prefFrag, bundle);//set the frag
                    } catch (Exception e) {
                        e.printStackTrace();
                        return false;
                    }
                    return true;
            }
            return false;
        }
    };

    /* #######################################################################
    onCreate is an autogenerated method for creating the activity.  However I did
    add a bit of code that automatically displays the home fragment when the activity
    is created.
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        BottomNavigationView navigation = (BottomNavigationView) findViewById(R.id.navigation);
        navigation.setOnNavigationItemSelectedListener(mOnNavigationItemSelectedListener);
        homeFragment homeFrag = new homeFragment();//new instance
        Bundle bundle = new Bundle();
        setFragment(homeFrag, bundle);//set the home frag as the default fragment displayed
    }

    /* ############################################
    onFragmentInteraction is an autogenerated method.
     */
    @Override
    public void onFragmentInteraction(Uri uri){
        //you can leave it empty
    }

    /* ################################################
    onEditButtonClicked is a necessary method for the editWorkoutFragment.
    This allows me to send the workoutname to changeWorkoutFragment.  So,
    MainActivity holds editWorkoutFragment.  Once a workout is selected and
    the user presses edit (or new since it uses the same method), this method
    listens for that button click and then starts the changeworkoutFragment.
     */
    public void onEditButtonClicked(String workoutName)
    {
        changeWorkoutFragment changefrag = new changeWorkoutFragment();
        Bundle bundle = new Bundle();
        bundle.putString("workoutName", workoutName);
        setFragment(changefrag, bundle);
    }

    /* #########################################################
    setFragment is a helper method that is a modular way to switch fragments
    since a lot of fragment switching occurs.  This allows me to not repeat
    this long segment of code.
     */
    public void setFragment (Fragment frag, Bundle bundle)
    {
        frag.setArguments(bundle);//include any parameters if any
        fragmentManager = getSupportFragmentManager();
        transaction = fragmentManager.beginTransaction();
        fragment = fragmentManager.findFragmentById(R.id.content);//retrieve current fragment ID
        if (fragment == null)
            transaction.add(R.id.content, frag);//first fragment
        else
            transaction.replace(R.id.content, frag);//replace but keep previous fragment in memory
        transaction.addToBackStack(null);
        transaction.commit();
    }

    /* ################################################################
    setPrefFrag is a helper method very similar to setFragment.  However, I
    use a special preference fragment that needed to have its own method.
    Comments can be considered same as the above method.
     */
    public void setPrefFrag (PreferenceFragmentCompat frag, Bundle bundle)
    {
        frag.setArguments(bundle);
        fragmentManager = getSupportFragmentManager();
        transaction = fragmentManager.beginTransaction();
        fragment = fragmentManager.findFragmentById(R.id.content);
        if (fragment == null)
            transaction.add(R.id.content, frag);
        else
            transaction.replace(R.id.content, frag);
        transaction.addToBackStack(null);
        transaction.commit();
    }
}


