/*
ResultsActivity.java created by Alex Dowlding
 */
package com.example.alex.liftcoach;

import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.RatingBar;
import android.widget.TextView;
import android.widget.Toast;

public class ResultsActivity extends AppCompatActivity {

    TextView elapsedTime;
    RatingBar feedbackStars;
    Button doneButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_results);

        SetupEntities();//sets up on screen widgets

        String finalTime = getIntent().getStringExtra("chronoTime");//retrieves time from previous activity
        elapsedTime.append(finalTime);//display the final time

        doneButton.setEnabled(false);//don't want them clicking done yet..
        doneButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (feedbackStars.getRating() >= 4)
                    showIncreasePopUp();//They thought it wen't well, trigger increase weight popup
                else if (feedbackStars.getRating() <= 2)
                    showDecreasePopUp();//It was rough, trigger decrease weight popup
                else //it was a middle-of-the-road workout
                {
                    Toast.makeText(getBaseContext(), "Workout Logged", Toast.LENGTH_LONG).show();
                    Intent intent = new Intent(getBaseContext(), MainActivity.class);
                    startActivity(intent); //go back to home.
                }
            }
        });

        feedbackStars.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                doneButton.setEnabled(true);
                //now that they have rated workout, let them finish
                return false;//default return statement, unknown difference between T/F
            }
        });
    }

    /* ################################################################################
    showIncreasePopUp is called if the user rated their workout with 4 stars or more.  In
    this case, a popup is displayed inviting the user to increase their max weight values
    for squat and/or bench
     */
    private void showIncreasePopUp()
    {
        AlertDialog.Builder helpBuilder = new AlertDialog.Builder(this);//start building popup
        helpBuilder.setTitle("Nice!");
        helpBuilder.setMessage("It looks like this workout is getting easy for you! " +
                "If you feel ready, you can increase your max outs by 5 lbs here" +
                ". These max values dictate your auxillary lift weights.");

        LayoutInflater inflater = getLayoutInflater();
        View checkboxLayout = inflater.inflate(R.layout.results_popup, null);
        helpBuilder.setView(checkboxLayout);//link to it

        final CheckBox benchCheckbox = (CheckBox) checkboxLayout.findViewById(R.id.benchCheckbox);
        final CheckBox squatCheckbox = (CheckBox) checkboxLayout.findViewById(R.id.squatCheckbox);
        //link code to both .xml checkboxes

        helpBuilder.setPositiveButton("Ok",
                new DialogInterface.OnClickListener() {
                //ok button has been made
                    public void onClick(DialogInterface dialog, int which) {
                        if (benchCheckbox.isChecked())//they want to increase bench
                        {
                            SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
                            //get those shared preferences again
                            String maxString = prefs.getString("benchMax", "135");
                            //get the current max weight for bench
                            int max = Integer.parseInt(maxString);
                            SharedPreferences.Editor editor = prefs.edit();
                            //allows editing of preferences
                            max += 5;
                            String newMaxString = Integer.toString(max);
                            editor.putString("benchMax", newMaxString);
                            //apply the new bench weight and save it
                            editor.commit();
                        }

                        if (squatCheckbox.isChecked())//not else if because both can be checked
                        {
                            SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
                            //retrieve shared preferences
                            String maxString = prefs.getString("squatMax", "135");
                            //get current max weight for squat
                            int max = Integer.parseInt(maxString);
                            SharedPreferences.Editor editor = prefs.edit();
                            //allows editing of preferences
                            max += 5;
                            String newMaxString = Integer.toString(max);
                            editor.putString("squatMax", newMaxString);
                            //apply the new squat weight and save it
                            editor.commit();
                        }
                        Toast.makeText(getBaseContext(), "Workout Logged", Toast.LENGTH_LONG).show();
                        Intent intent = new Intent(getBaseContext(), MainActivity.class);
                        startActivity(intent);//automatically go back to the home screen
                    }
                });

        AlertDialog helpDialog = helpBuilder.create();
        helpDialog.show();//show the dialog
        //remember, the above code is stored in a click listener, so this method
        //does not execute top down
    }

    /* ##############################################################################
    showDecreasePopUp, similarly to showIncreasePopUp, appears if the user rates their
    workout low.  This allows them to decrease their max values if they so choose.
    The same comments in the showIncreasePopUp method can be applied to this method.
    Only, the weight will decrease and not increase.
     */
    private void showDecreasePopUp()
    {
        AlertDialog.Builder helpBuilder = new AlertDialog.Builder(this);
        helpBuilder.setTitle("Oops!");
        helpBuilder.setMessage("It looks like this workout was a little too difficult! " +
                "Would you like to decrease any of your max values by 5 lbs?");

        LayoutInflater inflater = getLayoutInflater();
        View checkboxLayout = inflater.inflate(R.layout.results_popup, null);
        helpBuilder.setView(checkboxLayout);

        final CheckBox benchCheckbox = (CheckBox) checkboxLayout.findViewById(R.id.benchCheckbox);
        final CheckBox squatCheckbox = (CheckBox) checkboxLayout.findViewById(R.id.squatCheckbox);

        helpBuilder.setPositiveButton("Ok",
                new DialogInterface.OnClickListener() {

                    public void onClick(DialogInterface dialog, int which) {
                        // Do nothing but close the dialog
                        //MODIFY THE MAX OUT VALUES HERE I THINK
                        if (benchCheckbox.isChecked())
                        {
                            SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
                            String maxString = prefs.getString("benchMax", "135");
                            int max = Integer.parseInt(maxString);

                            SharedPreferences.Editor editor = prefs.edit();
                            max -= 5;
                            String newMaxString = Integer.toString(max);
                            editor.putString("benchMax", newMaxString);
                            editor.commit();
                        }

                        if (squatCheckbox.isChecked())
                        {
                            SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
                            String maxString = prefs.getString("squatMax", "135");
                            int max = Integer.parseInt(maxString);

                            SharedPreferences.Editor editor = prefs.edit();
                            max -= 5;
                            String newMaxString = Integer.toString(max);
                            editor.putString("squatMax", newMaxString);
                            editor.commit();
                        }

                        Toast.makeText(getBaseContext(), "Workout Logged", Toast.LENGTH_LONG).show();
                        Intent intent = new Intent(getBaseContext(), MainActivity.class);
                        startActivity(intent);
                    }
                });

        AlertDialog helpDialog = helpBuilder.create();
        helpDialog.show();
    }

    private void SetupEntities()
    {
        elapsedTime = (TextView) findViewById(R.id.elapsedTimeTextView);
        feedbackStars = (RatingBar) findViewById(R.id.feedbackStarsRatingBar);
        doneButton = (Button) findViewById(R.id.resultsDoneButton);
    }
}
