/*
changeWorkoutFragment.java created by Alex Dowlding
Acts as the edit screen for a particular workout.
 */
package com.example.alex.liftcoach;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import android.content.res.Resources;
import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.NumberPicker;
import android.widget.Toast;
import static android.content.Context.MODE_APPEND;
import static android.content.Context.MODE_PRIVATE;

/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link changeWorkoutFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * Use the {@link changeWorkoutFragment#newInstance} factory method to
 * create an instance of this fragment.
 */

public class changeWorkoutFragment extends Fragment {
    // TODO: Rename parameter arguments, choose names that match
    // the fragment initialization parameters, e.g. ARG_ITEM_NUMBER
    private static final String ARG_PARAM1 = "param1";
    private static final String ARG_PARAM2 = "param2";
    private String mParam1;
    private String mParam2;
    //Nervous to touch these so I left them

    private String filename;
    private String tempfilename = "tempfile";
    private String workoutTitleMarker = "#####";

    NumberPicker setsNumberPicker;
    NumberPicker repsNumberPicker;
    ListView masterLiftListView;
    ListView userWorkoutListView;
    ArrayAdapter<String> masterAdapter;
    String[] allLifts;
    String[] userLifts;
    ArrayAdapter<String> userAdapter;
    String previousWorkoutName;
    ImageButton saveButton;
    List<String> userList;
    EditText editText;

    private OnFragmentInteractionListener mListener;
    View view;

    public changeWorkoutFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param param1 Parameter 1.
     * @param param2 Parameter 2.
     * @return A new instance of fragment changeWorkoutFragment.
     */
    // TODO: Rename and change types and number of parameters
    public static changeWorkoutFragment newInstance(String param1, String param2) {
        changeWorkoutFragment fragment = new changeWorkoutFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PARAM1, param1);
        args.putString(ARG_PARAM2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            mParam1 = getArguments().getString(ARG_PARAM1);
            mParam2 = getArguments().getString(ARG_PARAM2);
        }
    }

    /* ##################################################################################################
    onCreateView is called when the fragment starts taking visual shape.  I call many of my setup functions here.
    Other setup code and listeners that didn't make sense to compartmentalize also are housed here
     */
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        view = inflater.inflate(R.layout.fragment_change_workout, container, false);

        SetupEntities(); //initializes all on screen entities

        filename = getString(R.string.filename); //retrieves filename from strings.xml for easy changing

        Resources res = getResources(); //allows string arrays to be taken
        allLifts = res.getStringArray(R.array.masterLiftListArray);//load array from resources
        userLifts = res.getStringArray(R.array.emptyArray);//load array from resources into local
        userList = new ArrayList<String>(Arrays.asList(userLifts));//now put userLifts into a list

        masterAdapter = new ArrayAdapter<String>(getContext(),android.R.layout.simple_list_item_multiple_choice, allLifts);
        //Above adapter needed to create list of checked TextViews.
        masterLiftListView.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE);
        masterLiftListView.setAdapter(masterAdapter);//ListView is now connected to adapter which is connected to the array.

        userAdapter = new ArrayAdapter<String>(getContext(), android.R.layout.simple_list_item_1, userList);
        //Above adapter needed to dynamically change list.
        userWorkoutListView.setAdapter(userAdapter);//Connect listview to adapter which is connected to userlist.

        Bundle bundle = this.getArguments();//get parameters from activity
        String workoutName = bundle.getString("workoutName");//retrieve workoutname
        previousWorkoutName = workoutName;//necessary for saving/writing
        editText.setText(workoutName);//set it on screen

        String[] setsArray = res.getStringArray(R.array.numberOfSetsArray);
        //Get number of sets from file
        setsNumberPicker.setDisplayedValues(setsArray);
        setsNumberPicker.setMinValue(1);
        setsNumberPicker.setMaxValue(setsArray.length);
        setsNumberPicker.setWrapSelectorWheel(false);

        String[] repsArray = res.getStringArray(R.array.numberOfRepsArray);
        //Get number of reps from file
        repsNumberPicker.setDisplayedValues(repsArray);
        repsNumberPicker.setMinValue(1);
        repsNumberPicker.setMaxValue(repsArray.length);
        repsNumberPicker.setWrapSelectorWheel(false);

        ReadFromFile(userList, setsNumberPicker, repsNumberPicker);
        //Retrieve whichever workout the user had selected in editWorkoutFragment

        masterLiftListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                String dataString = (String)parent.getItemAtPosition(position);//find the item checked
                if (masterLiftListView.isItemChecked(position))
                {
                    userList.add(dataString);
                    userAdapter.notifyDataSetChanged();
                    //add item if checked and notify
                }
                else
                {
                    userList.remove(dataString);
                    userAdapter.notifyDataSetChanged();
                    //else remove, unchecking is handled by GUI
                }
            }
        });

        saveButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                WriteToFile(userList, setsNumberPicker, repsNumberPicker);//save it
            }
        });

        return view;
    }

    /* ############################################################
    onAttach is a required default method that was autogenerated.  Connects
    fragment to the activity that called it.
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
    }

    /* #################################
    onDetach is a required default method that
    was autogenerated. Disconnects fragment from
    activity.
     */
    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */


    /* ####################################################
    OnFragmentInteractionListener had to be created manually upon
    creation of the fragment class.  This default barebones method works fine.
    If not created the app will crash when trying to attach fragment.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    /* #################################################################################################
    WriteToFile saves the user's list of lifts and number of sets/reps to file under the workout name
    they specify in the workoutname edittext.  Either adds a new workout to the file if the previous workoutname
    can't be found or amends the existing workout/workoutname with the new information.
     */
    private void WriteToFile(List<String> userList, NumberPicker setsPicker, NumberPicker repsPicker)
    {
        boolean existingWorkout = false; //assume it is new until proven otherwise.
        String name = editText.getText().toString();//get the workoutname

        if (name.contentEquals("NewWorkout"))//reserve this for new workouts templates
        {
            Toast.makeText(getContext(), "Please change workout name", Toast.LENGTH_LONG).show();//popup
            return;
        }

        File fileName = getContext().getFileStreamPath(filename);//get the main storage file
        File tempFile = getContext().getFileStreamPath(tempfilename);//get the tempfile that is overwritten
        //Tempfile is needed for workouts of various numbers of lifts.  Editing an existing workout
        //by adding/removing lifts would overwrite other portions of the file.  Therefore, 2 files needed.

        try {
            String line;
            FileOutputStream fileTempOutputStream = getContext().openFileOutput(tempfilename, MODE_PRIVATE);
            //by default, mode_private is private and overwrites all contents of file
            FileOutputStream fileOutputStream = getContext().openFileOutput(filename, MODE_PRIVATE | MODE_APPEND);
            //keeps file private but appends as well instead of erasing everything
            FileInputStream fileInputStream = getActivity().openFileInput(filename);
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);

            while ((line = bufferedReader.readLine()) != null)//if not end of file essentially
            {
                //Looking for previous workout name.  This was saved when file was read
                //if we find it then we know what to edit.  Otherwise probably a new workout
                //ALL WRITES OCCUR TO TEMP FILE
                if (line.toString().contentEquals(previousWorkoutName))// == didn't work
                {
                    fileTempOutputStream.write(name.getBytes());//write the new name - may match prev
                    previousWorkoutName = name; //prepping for next save operation
                    fileTempOutputStream.write('\n');
                    fileTempOutputStream.write((Integer.toString(setsPicker.getValue())).getBytes());//write sets
                    fileTempOutputStream.write('\n');
                    fileTempOutputStream.write((Integer.toString(repsPicker.getValue())).getBytes());//write reps

                    for (int i = 0; i < userList.size(); i++)
                    {
                        fileTempOutputStream.write('\n');
                        fileTempOutputStream.write(userList.get(i).getBytes());//write the lift
                    }

                    fileTempOutputStream.write('\n');
                    existingWorkout = true; //make sure it isn't counted as a new workout

                    while (line != null && !line.contentEquals(workoutTitleMarker))
                        line = bufferedReader.readLine(); //necessary so if taking workouts OFF of list
                    //the changes are accurate - otherwise it would copy/duplicate workouts before save

                    if (line != null)
                    {
                        //there is another workout after this one and a marker must be placed
                        fileTempOutputStream.write(workoutTitleMarker.getBytes());
                        fileTempOutputStream.write('\n');
                    }
                }
                else
                {
                    //copy line for line to the temp file.
                    fileTempOutputStream.write(line.getBytes());
                    fileTempOutputStream.write('\n'); //only if it's not the end of file...
                }
            }
            fileInputStream.close();

            if (!existingWorkout)//new workout
            {
                //ALL WRITES OCCUR TO "REAL" FILE
                fileOutputStream.write(workoutTitleMarker.getBytes());//write a new marker
                fileOutputStream.write('\n');
                fileOutputStream.write(name.getBytes());//write name
                fileOutputStream.write('\n');
                fileOutputStream.write((Integer.toString(setsPicker.getValue())).getBytes());//write sets
                fileOutputStream.write('\n');
                fileOutputStream.write((Integer.toString(repsPicker.getValue())).getBytes());//write reps
                for (int i = 0; i < userList.size(); i++)
                {
                    fileOutputStream.write('\n');
                    fileOutputStream.write(userList.get(i).getBytes());//write each lift
                }
                fileOutputStream.write('\n');
            }

            fileInputStream.close();
            fileOutputStream.close();
            fileTempOutputStream.close();

            if (existingWorkout)
            {
                //Edits occured and were written to the temp file
                fileName.delete();
                tempFile.renameTo(fileName);
            }

            Toast.makeText(getContext(), "Changes Saved", Toast.LENGTH_LONG).show();//popup

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    /* ############################################################################################
    ReadFromFile loads the workout into the userList, sets the appropriate checkboxes, and sets the correct
    numbers of sets and reps
     */
    public void ReadFromFile(List<String> userList, NumberPicker setsPicker, NumberPicker repsPicker)
    {
        try {
            userList.clear(); //no duplicates if back button pressed
            String Message;
            FileInputStream fileInputStream = getActivity().openFileInput(filename);
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            //StringBuffer stringBuffer = new StringBuffer();
            while ((Message = bufferedReader.readLine()) != null)//essentially end of file
            {
                if (Message.contentEquals(previousWorkoutName))//workout was found
                {
                    Message = bufferedReader.readLine();
                    setsPicker.setValue(Integer.parseInt(Message));//set the setspicker
                    Message = bufferedReader.readLine();
                    repsPicker.setValue(Integer.parseInt(Message));//set the repspicker
                    int index = 0;
                    Message = bufferedReader.readLine();
                    while(Message != null && !Message.contentEquals(workoutTitleMarker))
                    {
                        userList.add(index, Message);//add each lift
                        FindAndCheck(masterLiftListView, Message);//checkmark it on the master list
                        Message = bufferedReader.readLine();
                    }
                }
            }
            userAdapter.notifyDataSetChanged();//redraw list
        } catch (IOException e1) {
            e1.printStackTrace();
        }
    }

    /* ############################################################################
           FindAndCheck will ensure that each lift that is included in the userlist is also
           checked on the masterlist.
         */
    private void FindAndCheck(ListView listView, String lift)
    {
        for (int i = 0; i < listView.getCount(); i++)
        {
            //Check masterList for the lift and check it as it is contained in the user list
            if(listView.getItemAtPosition(i).toString().contentEquals(lift))
            listView.setItemChecked(i, true);
        }

    }

    /* ############################################################################
       SetupEntities initializes variable names for each on-screen element based off
       of the fragment_change_workout.xml.  Allows element manipulation by code.
     */
    private void SetupEntities()
    {
        masterLiftListView = (ListView) view.findViewById(R.id.masterLiftListView);
        userWorkoutListView = (ListView) view.findViewById(R.id.workoutLiftListView);
        editText = (EditText) view.findViewById(R.id.workoutNameText);
        setsNumberPicker = (NumberPicker) view.findViewById(R.id.setsNumberPicker);
        repsNumberPicker = (NumberPicker) view.findViewById(R.id.repsNumberPicker);
        saveButton = (ImageButton) view.findViewById(R.id.saveButton);
    }
}

